@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-'''A VCFv4.0 parser for Python.
+'''A VCFv4.0 and 4.1 parser for Python.
 
 Online version of PyVCF documentation is available at http://pyvcf.rtfd.org/
 
@@ -15,14 +15,14 @@ There main interface is the class: ``Reader``.  It takes a file-like
 object and acts as a reader::
 
     >>> import vcf
-    >>> vcf_reader = vcf.Reader(open('test/example-4.0.vcf', 'rb'))
+    >>> vcf_reader = vcf.Reader(open('vcf/test/example-4.0.vcf', 'r'))
     >>> for record in vcf_reader:
     ...     print record
-    Record(CHROM=20, POS=14370, REF=G, ALT=['A'])
-    Record(CHROM=20, POS=17330, REF=T, ALT=['A'])
-    Record(CHROM=20, POS=1110696, REF=A, ALT=['G', 'T'])
+    Record(CHROM=20, POS=14370, REF=G, ALT=[A])
+    Record(CHROM=20, POS=17330, REF=T, ALT=[A])
+    Record(CHROM=20, POS=1110696, REF=A, ALT=[G, T])
     Record(CHROM=20, POS=1230237, REF=T, ALT=[None])
-    Record(CHROM=20, POS=1234567, REF=GTCT, ALT=['G', 'GTACT'])
+    Record(CHROM=20, POS=1234567, REF=GTCT, ALT=[G, GTACT])
 
 
 This produces a great deal of information, but it is conveniently accessed.
@@ -50,12 +50,12 @@ one-entry Python lists (see, e.g., ``Record.ALT``).  Semicolon-delimited lists
 of key=value pairs are converted to Python dictionaries, with flags being given
 a ``True`` value. Integers and floats are handled exactly as you'd expect::
 
-    >>> vcf_reader = vcf.Reader(open('test/example-4.0.vcf', 'rb'))
+    >>> vcf_reader = vcf.Reader(open('vcf/test/example-4.0.vcf', 'r'))
     >>> record = vcf_reader.next()
     >>> print record.POS
     14370
     >>> print record.ALT
-    ['A']
+    [A]
     >>> print record.INFO['AF']
     [0.5]
 
@@ -69,7 +69,7 @@ examine properties of interest::
     >>> print record.nucl_diversity, record.aaf
     0.6 0.5
     >>> print record.get_hets()
-    [Call(sample=NA00002, GT=1|0, GQ=48)]
+    [Call(sample=NA00002, GT=1|0, HQ=[51, 51], DP=8, GQ=48)]
     >>> print record.is_snp, record.is_indel, record.is_transition, record.is_deletion
     True False True False
     >>> print record.var_type, record.var_subtype
@@ -98,7 +98,7 @@ call data in ``data``::
 
      >>> call = record.genotype('NA00001')
      >>> print call.site
-     Record(CHROM=20, POS=17330, REF=T, ALT=['A'])
+     Record(CHROM=20, POS=17330, REF=T, ALT=[A])
      >>> print call.sample
      NA00001
      >>> print call.data
@@ -113,7 +113,6 @@ There are also a number of methods::
     >>> print call.called, call.gt_type, call.gt_bases, call.phased
     True 0 T|T True
 
-
 Metadata regarding the VCF file itself can be investigated through the
 following attributes:
 
@@ -130,30 +129,40 @@ For example::
     >>> vcf_reader.samples
     ['NA00001', 'NA00002', 'NA00003']
     >>> vcf_reader.filters
-    {'q10': Filter(id='q10', desc='Quality below 10'), 's50': Filter(id='s50', desc='Less than 50% of samples have data')}
+    OrderedDict([('q10', Filter(id='q10', desc='Quality below 10')), ('s50', Filter(id='s50', desc='Less than 50% of samples have data'))])
     >>> vcf_reader.infos['AA'].desc
     'Ancestral Allele'
 
+ALT records are actually classes, so that you can interrogate them::
+
+    >>> reader = vcf.Reader(open('vcf/test/example-4.1-bnd.vcf'))
+    >>> _ = reader.next(); row = reader.next()
+    >>> print row
+    Record(CHROM=1, POS=2, REF=T, ALT=[T[2:3[])
+    >>> bnd = row.ALT[0]
+    >>> print bnd.withinMainAssembly, bnd.orientation, bnd.remoteOrientation, bnd.connectingSequence
+    True False True T
+
 Random access is supported for files with tabix indexes.  Simply call fetch for the
 region you are interested in::
 
-    >>> vcf_reader = vcf.Reader(filename='test/tb.vcf.gz')
-    >>> for record in vcf_reader.fetch('20', 1110696, 1230237):
+    >>> vcf_reader = vcf.Reader(filename='vcf/test/tb.vcf.gz')
+    >>> for record in vcf_reader.fetch('20', 1110696, 1230237):  # doctest: +SKIP
     ...     print record
-    Record(CHROM=20, POS=1110696, REF=A, ALT=['G', 'T'])
+    Record(CHROM=20, POS=1110696, REF=A, ALT=[G, T])
     Record(CHROM=20, POS=1230237, REF=T, ALT=[None])
 
 Or extract a single row::
 
-    >>> print vcf_reader.fetch('20', 1110696)
-    Record(CHROM=20, POS=1110696, REF=A, ALT=['G', 'T'])
+    >>> print vcf_reader.fetch('20', 1110696)  # doctest: +SKIP
+    Record(CHROM=20, POS=1110696, REF=A, ALT=[G, T])
 
 
 The ``Writer`` class provides a way of writing a VCF file.  Currently, you must specify a
 template ``Reader`` which provides the metadata::
 
-    >>> vcf_reader = vcf.Reader(filename='test/tb.vcf.gz')
-    >>> vcf_writer = vcf.Writer(file('/dev/null', 'w'), vcf_reader)
+    >>> vcf_reader = vcf.Reader(filename='vcf/test/tb.vcf.gz')
+    >>> vcf_writer = vcf.Writer(open('/dev/null', 'w'), vcf_reader)
     >>> for record in vcf_reader:
     ...     vcf_writer.write_record(record)
 
@@ -163,9 +172,9 @@ declared by other packages will be available for use in this script.  Please
 see :doc:`FILTERS` for full description.
 
 '''
-from parser import Reader, Writer
-from parser import VCFReader, VCFWriter
-from filters import Base as Filter
-from parser import RESERVED_INFO, RESERVED_FORMAT
+from vcf.parser import Reader, Writer
+from vcf.parser import VCFReader, VCFWriter
+from vcf.filters import Base as Filter
+from vcf.parser import RESERVED_INFO, RESERVED_FORMAT
 
-VERSION = '0.4.6'
+VERSION = '0.5.0'
