@@ -1,4 +1,3 @@
-
 import collections
 import re
 import csv
@@ -23,21 +22,155 @@ RESERVED_INFO = {
     'AA': 'String', 'AC': 'Integer', 'AF': 'Float', 'AN': 'Integer',
     'BQ': 'Float', 'CIGAR': 'String', 'DB': 'Flag', 'DP': 'Integer',
     'END': 'Integer', 'H2': 'Flag', 'MQ': 'Float', 'MQ0': 'Integer',
-    'NS': 'Integer', 'SB': 'String', 'SOMATIC': 'Flag', 'VALIDATED': 'Flag'
+    'NS': 'Integer', 'SB': 'String', 'SOMATIC': 'Flag', 'VALIDATED': 'Flag',
+
+    # VCF 4.1 Additions
+    'IMPRECISE':'Flag', 'NOVEL':'Flag', 'END':'Integer', 'SVTYPE':'String',
+    'CIPOS':'Integer','CIEND':'Integer','HOMLEN':'Integer','HOMSEQ':'Integer',
+    'BKPTID':'String','MEINFO':'String','METRANS':'String','DGVID':'String',
+    'DBVARID':'String','MATEID':'String','PARID':'String','EVENT':'String',
+    'CILEN':'Integer','CN':'Integer','CNADJ':'Integer','CICN':'Integer',
+    'CICNADJ':'Integer'
 }
 
 RESERVED_FORMAT = {
     'GT': 'String', 'DP': 'Integer', 'FT': 'String', 'GL': 'Float',
-    'GQ': 'Float', 'HQ': 'Float'
+    'GQ': 'Float', 'HQ': 'Float',
+
+    # VCF 4.1 Additions
+    'CN':'Integer','CNQ':'Float','CNL':'Float','NQ':'Integer','HAP':'Integer',
+    'AHAP':'Integer'
+}
+
+# Spec is a bit weak on which metadata lines are singular, like fileformat
+# and which can have repeats, like contig
+SINGULAR_METADATA = ['fileformat', 'fileDate', 'reference']
+
+# Conversion between value in file and Python value
+field_counts = {
+    '.': None,  # Unknown number of values
+    'A': -1,  # Equal to the number of alleles in a given record
+    'G': -2,  # Equal to the number of genotypes in a given record
 }
 
 
 _Info = collections.namedtuple('Info', ['id', 'num', 'type', 'desc'])
 _Filter = collections.namedtuple('Filter', ['id', 'desc'])
+_Alt = collections.namedtuple('Alt', ['id', 'desc'])
 _Format = collections.namedtuple('Format', ['id', 'num', 'type', 'desc'])
 _SampleInfo = collections.namedtuple('SampleInfo', ['samples', 'gt_bases', 'gt_types', 'gt_phases'])
 
 
+class _AltRecord(object):
+    '''An alternative allele record: either replacement string, SV placeholder, or breakend'''
+
+    def __init__(self, type):
+        #: String to describe the type of variant, by default "SNV" or "MNV", but can be extended to any of the types described in the ALT lines of the header (e.g. "DUP", "DEL", "INS"...)
+        self.type = type
+
+    def __str__(self):
+        assert False, "_AltRecord is an abstract class, you should be using a subclass instead"
+
+    def __eq__(self, other):
+        return self.type == other.type
+
+
+class _Substitution(_AltRecord):
+    '''A basic ALT record, where a REF sequence is replaced by an ALT sequence'''
+
+    def __init__(self, nucleotides):
+        if len(nucleotides) == 1:
+            super(_Substitution, self).__init__("SNV")
+        else:
+            super(_Substitution, self).__init__("MNV")
+        #: Alternate sequence
+        self.sequence = str(nucleotides)
+
+    def __str__(self):
+        return self.sequence
+
+    def __repr__(self):
+        return str(self)
+
+    def __len__(self):
+        return len(self.sequence)
+
+    def __eq__(self, other):
+        if isinstance(other, basestring):
+            return self.sequence == other
+        else:
+            return super(_Substitution, self).__eq__(other) and self.sequence == other.sequence
+
+
+class _Breakend(_AltRecord):
+    '''A breakend which is paired to a remote location on or off the genome'''
+
+    def __init__(self, chr, pos, orientation, remoteOrientation, connectingSequence, withinMainAssembly):
+        super(_Breakend, self).__init__("BND")
+        #: The chromosome of breakend's mate.
+        self.chr = str(chr)
+        #: The coordinate of breakend's mate.
+        self.pos = int(pos)
+        #: The orientation of breakend's mate. If the sequence 3' of the breakend's mate is connected, True, else if the sequence 5' of the breakend's mate is connected, False.
+        self.remoteOrientation = remoteOrientation
+        #: If the breakend mate is within the assembly, True, else False if the breakend mate is on a contig in an ancillary assembly file.
+        self.withinMainAssembly = withinMainAssembly
+        #: The orientation of breakend. If the sequence 3' of the breakend is connected, True, else if the sequence 5' of the breakend is connected, False.
+        self.orientation = orientation
+        #: The breakpoint's connecting sequence.
+        self.connectingSequence = connectingSequence
+
+    def __repr__(self):
+        return str(self)
+
+    def __str__(self):
+        if self.chr is None:
+            remoteTag = '.'
+        else:
+            if self.withinMainAssembly:
+                remoteChr = self.chr
+            else:
+                remoteChr = "<" + self.chr + ">"
+            if self.remoteOrientation:
+                remoteTag = "[" + remoteChr + ":" + str(self.pos) + "["
+            else:
+                remoteTag = "]" + remoteChr + ":" + str(self.pos) + "]"
+
+        if self.orientation:
+            return remoteTag + self.connectingSequence
+        else:
+            return self.connectingSequence + remoteTag
+
+    def __eq__(self, other):
+        return super(_Breakend, self).__eq__(other) \
+                and self.chr == other.chr \
+                and self.pos == other.pos \
+                and self.remoteOrientation == other.remoteOrientation \
+                and self.withinMainAssembly == other.withinMainAssembly \
+                and self.orientation == other.orientation \
+                and self.connectingSequence == other.connectingSequence
+
+
+class _SingleBreakend(_Breakend):
+    '''A single breakend'''
+
+    def __init__(self, orientation, connectingSequence):
+        super(_SingleBreakend, self).__init__(None, None, orientation, None, connectingSequence, None)
+
+
+class _SV(_AltRecord):
+    '''An SV placeholder'''
+
+    def __init__(self, type):
+        super(_SV, self).__init__(type)
+
+    def __str__(self):
+        return "<" + self.type + ">"
+
+    def __repr__(self):
+        return str(self)
+
+
 class _vcf_metadata_parser(object):
     '''Parse the metadat in the header of a VCF file.'''
     def __init__(self):
@@ -52,13 +185,25 @@ class _vcf_metadata_parser(object):
             ID=(?P<id>[^,]+),
             Description="(?P<desc>[^"]*)"
             >''', re.VERBOSE)
+        self.alt_pattern = re.compile(r'''\#\#ALT=<
+            ID=(?P<id>[^,]+),
+            Description="(?P<desc>[^"]*)"
+            >''', re.VERBOSE)
         self.format_pattern = re.compile(r'''\#\#FORMAT=<
             ID=(?P<id>.+),
             Number=(?P<number>-?\d+|\.|[AG]),
             Type=(?P<type>.+),
             Description="(?P<desc>.*)"
             >''', re.VERBOSE)
-        self.meta_pattern = re.compile(r'''##(?P<key>.+)=(?P<val>.+)''')
+        self.meta_pattern = re.compile(r'''##(?P<key>.+?)=(?P<val>.+)''')
+
+    def vcf_field_count(self, num_str):
+        """Cast vcf header numbers to integer or None"""
+        if num_str not in field_counts:
+            # Fixed, specified number
+            return int(num_str)
+        else:
+            return field_counts[num_str]
 
     def read_info(self, info_string):
         '''Read a meta-information INFO line.'''
@@ -67,12 +212,7 @@ class _vcf_metadata_parser(object):
             raise SyntaxError(
                 "One of the INFO lines is malformed: %s" % info_string)
 
-        try:
-            num = int(match.group('number'))
-            if num < 0:
-                num = None
-        except ValueError:
-            num = None
+        num = self.vcf_field_count(match.group('number'))
 
         info = _Info(match.group('id'), num,
                      match.group('type'), match.group('desc'))
@@ -90,6 +230,17 @@ class _vcf_metadata_parser(object):
 
         return (match.group('id'), filt)
 
+    def read_alt(self, alt_string):
+        '''Read a meta-information ALTline.'''
+        match = self.alt_pattern.match(alt_string)
+        if not match:
+            raise SyntaxError(
+                "One of the FILTER lines is malformed: %s" % alt_string)
+
+        alt = _Alt(match.group('id'), match.group('desc'))
+
+        return (match.group('id'), alt)
+
     def read_format(self, format_string):
         '''Read a meta-information FORMAT line.'''
         match = self.format_pattern.match(format_string)
@@ -97,21 +248,27 @@ class _vcf_metadata_parser(object):
             raise SyntaxError(
                 "One of the FORMAT lines is malformed: %s" % format_string)
 
-        try:
-            num = int(match.group('number'))
-            if num < 0:
-                num = None
-        except ValueError:
-            num = None
+        num = self.vcf_field_count(match.group('number'))
 
         form = _Format(match.group('id'), num,
                        match.group('type'), match.group('desc'))
 
         return (match.group('id'), form)
 
+    def read_meta_hash(self, meta_string):
+        items = re.split("[<>]", meta_string)
+        # Removing initial hash marks and final equal sign
+        key = items[0][2:-1]
+        hashItems = items[1].split(',')
+        val = dict(item.split("=") for item in hashItems)
+        return key, val
+
     def read_meta(self, meta_string):
-        match = self.meta_pattern.match(meta_string)
-        return match.group('key'), match.group('val')
+        if re.match("##.+=<", meta_string):
+            return self.read_meta_hash(meta_string)
+        else:
+            match = self.meta_pattern.match(meta_string)
+            return match.group('key'), match.group('val')
 
 
 class _Call(object):
@@ -126,12 +283,13 @@ class _Call(object):
         self.sample = sample
         #: Dictionary of data from the VCF file
         self.data = data
-        self.gt_nums = self.data['GT']
+        self.gt_nums = self.data.get('GT')
         #: True if the GT is not ./.
         self.called = self.gt_nums is not None
 
     def __repr__(self):
-        return "Call(sample=%s, GT=%s, GQ=%s)" % (self.sample, self.gt_nums, self.data.get('GQ', ''))
+        return "Call(sample=%s, GT=%s%s)" % (self.sample, self.gt_nums,
+                "".join([", %s=%s" % (X, self.data[X]) for X in self.data if X != 'GT']))
 
     def __eq__(self, other):
         """ Two _Calls are equal if their _Records are equal
@@ -141,6 +299,15 @@ class _Call(object):
                 and self.sample == other.sample
                 and self.gt_type == other.gt_type)
 
+    def gt_phase_char(self):
+        return "/" if not self.phased else "|"
+
+    @property
+    def gt_alleles(self):
+        '''The numbers of the alleles called at a given sample'''
+        # grab the numeric alleles of the gt string; tokenize by phasing
+        return self.gt_nums.split(self.gt_phase_char())
+
     @property
     def gt_bases(self):
         '''The actual genotype alleles.
@@ -148,14 +315,9 @@ class _Call(object):
         '''
         # nothing to do if no genotype call
         if self.called:
-            # grab the numeric alleles of the gt string; tokenize by phasing
-            phase_char = "/" if not self.phased else "|"
-            (a1, a2) = self.gt_nums.split(phase_char)
             # lookup and return the actual DNA alleles
             try:
-                return self.site.alleles[int(a1)] + \
-                       phase_char + \
-                       self.site.alleles[int(a2)]
+                return self.gt_phase_char().join(str(self.site.alleles[int(X)]) for X in self.gt_alleles)
             except:
                 sys.stderr.write("Allele number not found in list of alleles\n")
         else:
@@ -171,11 +333,9 @@ class _Call(object):
         '''
         # extract the numeric alleles of the gt string
         if self.called:
-            # grab the numeric alleles of the gt string; tokenize by phasing
-            (a1, a2) = self.gt_nums.split("/") \
-                if not self.phased else self.gt_nums.split("|")
-            if a1 == a2:
-                if a1 == "0": return 0
+            alleles = self.gt_alleles
+            if all(X == alleles[0] for X in alleles[1:]):
+                if alleles[0] == "0": return 0
                 else: return 2
             else: return 1
         else: return None
@@ -185,7 +345,7 @@ class _Call(object):
         '''A boolean indicating whether or not
            the genotype is phased for this sample
         '''
-        return self.data['GT'] is not None and self.data['GT'].find("|") >= 0
+        return self.gt_nums is not None and self.gt_nums.find("|") >= 0
 
     def __getitem__(self, key):
         """ Lookup value, backwards compatibility """
@@ -313,8 +473,8 @@ class _Record(object):
         hom_ref = self.num_hom_ref
         het = self.num_het
         hom_alt = self.num_hom_alt
-        num_chroms = float(2.0*self.num_called)
-        return float(het + 2*hom_alt)/float(num_chroms)
+        num_chroms = float(2.0 * self.num_called)
+        return float(het + 2 * hom_alt) / float(num_chroms)
 
     @property
     def nucl_diversity(self):
@@ -332,9 +492,9 @@ class _Record(object):
         if len(self.ALT) > 1:
             return None
         p = self.aaf
-        q = 1.0-p
-        num_chroms = float(2.0*self.num_called)
-        return float(num_chroms/(num_chroms-1.0)) * (2.0 * p * q)
+        q = 1.0 - p
+        num_chroms = float(2.0 * self.num_called)
+        return float(num_chroms / (num_chroms - 1.0)) * (2.0 * p * q)
 
     def get_hom_refs(self):
         """ The list of hom ref genotypes"""
@@ -357,6 +517,8 @@ class _Record(object):
         """ Return whether or not the variant is a SNP """
         if len(self.REF) > 1: return False
         for alt in self.ALT:
+            if alt is None or alt.type != "SNV":
+                return False
             if alt not in ['A', 'C', 'G', 'T']:
                 return False
         return True
@@ -370,6 +532,8 @@ class _Record(object):
         for alt in self.ALT:
             if alt is None:
                 return True
+            if alt.type != "SNV" and alt.type != "MNV":
+                return False
             elif len(alt) != len(self.REF):
                 # the diff. b/w INDELs and SVs can be murky.
                 if not is_sv:
@@ -479,8 +643,7 @@ class _Record(object):
             elif self.is_sv_precise:
                 return self.INFO['SVTYPE']
             else:
-                # first remove both "<" and ">" from ALT
-                return self.ALT[0].strip('<>')
+                return self.ALT[0].type
         else:
             return "unknown"
 
@@ -539,12 +702,14 @@ class Reader(object):
             if sys.version > '3':
                 self.reader = codecs.getreader('ascii')(self.reader)
 
-        #: metadata fields from header
+        #: metadata fields from header (string or hash, depending)
         self.metadata = None
         #: INFO fields from header
         self.infos = None
         #: FILTER fields from header
         self.filters = None
+        #: ALT fields from header
+        self.alts = None
         #: FORMAT fields from header
         self.formats = None
         self.samples = None
@@ -563,7 +728,7 @@ class Reader(object):
 
         The end user shouldn't have to use this.  She can access the metainfo
         directly with ``self.metadata``.'''
-        for attr in ('metadata', 'infos', 'filters', 'formats'):
+        for attr in ('metadata', 'infos', 'filters', 'alts', 'formats'):
             setattr(self, attr, OrderedDict())
 
         parser = _vcf_metadata_parser()
@@ -581,17 +746,26 @@ class Reader(object):
                 key, val = parser.read_filter(line)
                 self.filters[key] = val
 
+            elif line.startswith('##ALT'):
+                key, val = parser.read_alt(line)
+                self.alts[key] = val
+
             elif line.startswith('##FORMAT'):
                 key, val = parser.read_format(line)
                 self.formats[key] = val
 
             else:
                 key, val = parser.read_meta(line.strip())
-                self.metadata[key] = val
+                if key in SINGULAR_METADATA:
+                    self.metadata[key] = val
+                else:
+                    if key not in self.metadata:
+                        self.metadata[key] = []
+                    self.metadata[key].append(val)
 
             line = self.reader.next()
 
-        fields = line.rstrip().split('\t')
+        fields = re.split('\t| +', line.rstrip())
         self.samples = fields[9:]
         self._sample_indexes = dict([(x,i) for (i,x) in enumerate(self.samples)])
 
@@ -634,7 +808,10 @@ class Reader(object):
             elif entry_type == 'Flag':
                 val = True
             elif entry_type == 'String':
-                val = entry[1]
+                try:
+                    val = entry[1]
+                except IndexError:
+                    val = True
 
             try:
                 if self.infos[ID].num == 1 and entry_type != 'String':
@@ -681,6 +858,7 @@ class Reader(object):
             samp_fmt = sf
 
         samp_data = []
+        _map = self._map
 
         for name, sample in itertools.izip(self.samples, samples):
 
@@ -713,9 +891,9 @@ class Reader(object):
                 vals = vals.split(',')
 
                 if entry_type == 'Integer':
-                    sampdict[fmt] = self._map(int, vals)
+                    sampdict[fmt] = _map(int, vals)
                 elif entry_type == 'Float' or entry_type == 'Numeric':
-                    sampdict[fmt] = self._map(float, vals)
+                    sampdict[fmt] = _map(float, vals)
                 else:
                     sampdict[fmt] = vals
 
@@ -725,10 +903,38 @@ class Reader(object):
 
         return samp_data
 
+    def parseALT(self, str):
+        if re.search('[\[\]]', str) is not None:
+            # Paired breakend
+            items = re.split('[\[\]]', str)
+            remoteCoords = items[1].split(':')
+            chr = remoteCoords[0]
+            if chr[0] == '<':
+                chr = chr[1:-1]
+                withinMainAssembly = False
+            else:
+                withinMainAssembly = True
+            pos = remoteCoords[1]
+            orientation = (str[0] == '[' or str[0] == ']')
+            remoteOrientation = (re.search('\[', str) is not None)
+            if orientation:
+                connectingSequence = items[2]
+            else:
+                connectingSequence = items[0]
+            return _Breakend(chr, pos, orientation, remoteOrientation, connectingSequence, withinMainAssembly)
+        elif str[0] == '.' and len(str) > 1:
+            return _SingleBreakend(True, str[1:])
+        elif str[-1] == '.' and len(str) > 1:
+            return _SingleBreakend(False, str[:-1])
+        elif str[0] == "<" and str[-1] == ">":
+            return _SV(str[1:-1])
+        else:
+            return _Substitution(str)
+
     def next(self):
         '''Return the next record in the file.'''
         line = self.reader.next()
-        row = line.split()
+        row = re.split('\t| +', line.strip())
         chrom = row[0]
         if self._prepend_chr:
             chrom = 'chr' + chrom
@@ -740,7 +946,7 @@ class Reader(object):
             ID = None
 
         ref = row[3]
-        alt = self._map(str, row[4].split(','))
+        alt = self._map(self.parseALT, row[4].split(','))
 
         try:
             qual = int(row[5])
@@ -790,7 +996,7 @@ class Reader(object):
         start = start - 1
 
         if end is None:
-            self.reader = self._tabix.fetch(chrom, start, start+1)
+            self.reader = self._tabix.fetch(chrom, start, start + 1)
             try:
                 return self.next()
             except StopIteration:
@@ -805,18 +1011,29 @@ class Writer(object):
 
     fixed_fields = "#CHROM POS ID REF ALT QUAL FILTER INFO FORMAT".split()
 
-    def __init__(self, stream, template):
-        self.writer = csv.writer(stream, delimiter="\t")
+    # Reverse keys and values in header field count dictionary
+    counts = dict((v,k) for k,v in field_counts.iteritems())
+
+    def __init__(self, stream, template, lineterminator="\r\n"):
+        self.writer = csv.writer(stream, delimiter="\t", lineterminator=lineterminator)
         self.template = template
 
-        for line in template.metadata.iteritems():
-            stream.write('##%s=%s\n' % line)
+        two = '##{key}=<ID={0},Description="{1}">\n'
+        four = '##{key}=<ID={0},Number={num},Type={2},Description="{3}">\n'
+        _num = self._fix_field_count
+        for (key, vals) in template.metadata.iteritems():
+            if key in SINGULAR_METADATA:
+                vals = [vals]
+            for val in vals:
+                stream.write('##{0}={1}\n'.format(key, val))
         for line in template.infos.itervalues():
-            stream.write('##INFO=<ID=%s,Number=%s,Type=%s,Description="%s">\n' % tuple(self._map(str, line)))
+            stream.write(four.format(key="INFO", *line, num=_num(line.num)))
         for line in template.formats.itervalues():
-            stream.write('##FORMAT=<ID=%s,Number=%s,Type=%s,Description="%s">\n' % tuple(self._map(str, line)))
+            stream.write(four.format(key="FORMAT", *line, num=_num(line.num)))
         for line in template.filters.itervalues():
-            stream.write('##FILTER=<ID=%s,Description="%s">\n' % tuple(self._map(str, line)))
+            stream.write(two.format(key="FILTER", *line))
+        for line in template.alts.itervalues():
+            stream.write(two.format(key="ALT", *line))
 
         self._write_header()
 
@@ -827,36 +1044,52 @@ class Writer(object):
     def write_record(self, record):
         """ write a record to the file """
         ffs = self._map(str, [record.CHROM, record.POS, record.ID, record.REF]) \
-              + [self._format_alt(record.ALT), record.QUAL or '.', record.FILTER or '.',
+              + [self._format_alt(record.ALT), record.QUAL or '.', self._format_filter(record.FILTER),
                  self._format_info(record.INFO), record.FORMAT]
 
         samples = [self._format_sample(record.FORMAT, sample)
             for sample in record.samples]
         self.writer.writerow(ffs + samples)
 
+    def _fix_field_count(self, num_str):
+        """Restore header number to original state"""
+        if num_str not in self.counts:
+            return num_str
+        else:
+            return self.counts[num_str]
+
     def _format_alt(self, alt):
-        return ','.join([x or '.' for x in alt])
+        return ','.join(self._map(str, alt))
+
+    def _format_filter(self, flt):
+        return self._stringify(flt, none='PASS', delim=';')
 
     def _format_info(self, info):
         if not info:
             return '.'
-        return ';'.join(["%s=%s" % (x, self._stringify(y)) for x, y in info.iteritems()])
+        return ';'.join([self._stringify_pair(x,y) for x, y in info.iteritems()])
 
     def _format_sample(self, fmt, sample):
         if sample.data["GT"] is None:
             return "./."
         return ':'.join(self._stringify(sample.data[f]) for f in fmt.split(':'))
 
-    def _stringify(self, x, none='.'):
+    def _stringify(self, x, none='.', delim=','):
         if type(x) == type([]):
-            return ','.join(self._map(str, x, none))
+            return delim.join(self._map(str, x, none))
         return str(x) if x is not None else none
 
+    def _stringify_pair(self, x, y, none='.', delim=','):
+        if y and isinstance(y, bool):
+            return str(x)
+        return "%s=%s" % (str(x), self._stringify(y, none=none, delim=delim))
+
     def _map(self, func, iterable, none='.'):
         '''``map``, but make None values none.'''
         return [func(x) if x is not None else none
                 for x in iterable]
 
+
 def __update_readme():
     import sys, vcf
     file('README.rst', 'w').write(vcf.__doc__)
