@@ -8,7 +8,7 @@ from StringIO import StringIO
 import vcf
 from vcf import utils
 
-suite = doctest.DocTestSuite(vcf.parser)
+suite = doctest.DocTestSuite(vcf)
 
 
 def fh(fname, mode='rt'):
@@ -44,39 +44,63 @@ class TestVcfSpecs(unittest.TestCase):
 
 
     def test_vcf_4_1(self):
-        return
         reader = vcf.Reader(fh('example-4.1.vcf'))
         self.assertEqual(reader.metadata['fileformat'],  'VCFv4.1')
 
         # contigs were added in vcf4.1
         # probably need to add a reader.contigs attribute
         assert 'contig' in reader.metadata
+        assert 'ID' in reader.metadata['contig'][0]
+        assert reader.metadata['contig'][0]['ID'] == '20'
 
         # test we can walk the file at least
         for r in reader:
             for c in r:
                 assert c
 
-        # asserting False while I work out what to check
-        assert False
-
     def test_vcf_4_1_sv(self):
-        return
-
         reader = vcf.Reader(fh('example-4.1-sv.vcf'))
 
         assert 'SVLEN' in reader.infos
+        assert 'fileDate' in reader.metadata
+        assert 'DEL' in reader.alts
 
         # test we can walk the file at least
         for r in reader:
             print(r)
+            for a in r.ALT:
+                print(a)
             for c in r:
                 print(c)
                 assert c
 
-        # asserting False while I work out what to check
-        assert False
+    def test_vcf_4_1_bnd(self):
+        reader = vcf.Reader(fh('example-4.1-bnd.vcf'))
 
+        # test we can walk the file at least
+        for r in reader:
+            print(r)
+            for a in r.ALT:
+                print(a)
+            if r.ID == "bnd1":
+                    assert len(r.ALT) == 1
+                    assert r.ALT[0].type == "BND"
+                    assert r.ALT[0].chr == "2"
+                    assert r.ALT[0].pos == 3
+                    assert r.ALT[0].orientation == False
+                    assert r.ALT[0].remoteOrientation == True
+                    assert r.ALT[0].connectingSequence == "T"
+            if r.ID == "bnd4":
+                    assert len(r.ALT) == 1
+                    assert r.ALT[0].type == "BND"
+                    assert r.ALT[0].chr == "1"
+                    assert r.ALT[0].pos == 2
+                    assert r.ALT[0].orientation == True
+                    assert r.ALT[0].remoteOrientation == False
+                    assert r.ALT[0].connectingSequence == "G"
+            for c in r:
+                print(c)
+                assert c
 
 class TestGatkOutput(unittest.TestCase):
 
@@ -136,10 +160,6 @@ class TestFreebayesOutput(TestGatkOutput):
 
     def testParse(self):
         reader = vcf.Reader(fh('freebayes.vcf'))
-
-
-
-
         print(reader.samples)
         self.assertEqual(len(reader.samples), 7)
         n = 0
@@ -158,6 +178,15 @@ class TestSamtoolsOutput(unittest.TestCase):
         self.assertEqual(sum(1 for _ in reader), 11)
 
 
+class TestBcfToolsOutput(unittest.TestCase):
+    def testParse(self):
+        reader = vcf.Reader(fh('bcftools.vcf'))
+        self.assertEqual(len(reader.samples), 1)
+        for r in reader:
+            for s in r.samples:
+                s.phased
+
+
 class Test1kg(unittest.TestCase):
 
     def testParse(self):
@@ -167,6 +196,14 @@ class Test1kg(unittest.TestCase):
         for _ in reader:
             pass
 
+    def test_issue_49(self):
+        """docstring for test_issue_49"""
+        reader = vcf.Reader(fh('issue_49.vcf', 'r'))
+
+        self.assertEqual(len(reader.samples), 0)
+        for _ in reader:
+            pass
+
 
 class TestWriter(unittest.TestCase):
 
@@ -246,6 +283,11 @@ class TestRecord(unittest.TestCase):
 
     def test_is_snp(self):
         reader = vcf.Reader(fh('example-4.0.vcf'))
+        for r in reader:
+            print(r)
+            for c in r:
+                print(c)
+                assert c
         for var in reader:
             is_snp = var.is_snp
             if var.POS == 14370:
@@ -697,6 +739,16 @@ class TestUtils(unittest.TestCase):
                 assert recs[0] is not None
                 assert recs[1] is not None
 
+    def test_trim(self):
+        tests = [('TAA GAA', 'T G'),
+                 ('TA TA', 'T T'),
+                 ('AGTTTTTA AGTTTA', 'AGTT AG'),
+                 ('TATATATA TATATA', 'TAT T'),
+                 ('TATATA TATATATA', 'T TAT'),
+                 ('ACCCCCCC ACCCCCCCCCC ACCCCCCCCC ACCCCCCCCCCC', 'A ACCC ACC ACCCC')]
+        for sequences, expected in tests:
+            self.assertEqual(utils.trim_common_suffix(*sequences.split()),
+                             expected.split())
 
 
 
@@ -712,3 +764,4 @@ suite.addTests(unittest.TestLoader().loadTestsFromTestCase(Test1kg))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestRecord))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestCall))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestRegression))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestVcfSpecs))
